package com.javainuse.boot_elasticsearch_crud.config;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.security.KeyStore;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.Base64;

import javax.net.ssl.SSLContext;

import org.apache.http.auth.AuthScope;
import org.apache.http.auth.UsernamePasswordCredentials;
import org.apache.http.client.CredentialsProvider;
import org.apache.http.impl.client.BasicCredentialsProvider;
import org.apache.http.impl.nio.client.HttpAsyncClientBuilder;
import org.apache.http.ssl.SSLContextBuilder;
import org.apache.http.ssl.SSLContexts;
import org.elasticsearch.client.RestClientBuilder.HttpClientConfigCallback;
import org.springframework.context.annotation.Configuration;

import com.azure.identity.DefaultAzureCredentialBuilder;
import com.azure.security.keyvault.secrets.SecretClient;
import com.azure.security.keyvault.secrets.SecretClientBuilder;
import com.azure.security.keyvault.secrets.models.KeyVaultSecret;

@Configuration
public class HttpClientConfigImpl implements HttpClientConfigCallback {

	@Override
	public HttpAsyncClientBuilder customizeHttpClient(HttpAsyncClientBuilder httpClientBuilder) {

		String keyVaultUrl = "https://javainuse.vault.azure.net/";

		try {

			// Create a SecretClient instance to interact with Azure Key Vault
			SecretClient secretClient = new SecretClientBuilder().vaultUrl(keyVaultUrl)
					.credential(new DefaultAzureCredentialBuilder().build()).buildClient();

			// Retrieve the elastic username secret from Azure Key Vault
			KeyVaultSecret elasticUsernameSecret = secretClient.getSecret("elastic-username");
			String elasticUsername = elasticUsernameSecret.getValue();

			// Retrieve the elastic password from Azure Key Vault
			KeyVaultSecret elasticPasswordSecret = secretClient.getSecret("elastic-password");
			String elasticPassword = elasticPasswordSecret.getValue();

			final CredentialsProvider credentialsProvider = new BasicCredentialsProvider();
			UsernamePasswordCredentials usernamePasswordCredentials = new UsernamePasswordCredentials(elasticUsername,
					elasticPassword);
			credentialsProvider.setCredentials(AuthScope.ANY, usernamePasswordCredentials);
			httpClientBuilder.setDefaultCredentialsProvider(credentialsProvider);

			String trustLocationStore = "E:\\trial\\elasticsearch-8.13.4-windows-x86_64\\elasticsearch-8.13.4\\config\\certs\\truststore.p12";

			// Retrieve the Base64-encoded secret from Azure Key Vault
			KeyVaultSecret certSecret = secretClient.getSecret("cert");
			String base64EncodedCert = certSecret.getValue();

			// Decode the Base64-encoded string to a certificate
			byte[] certificateBytes = Base64.getDecoder().decode(base64EncodedCert);

			// Retrieve the truststore password secret from Azure Key Vault
			KeyVaultSecret truststorePasswordSecret = secretClient.getSecret("truststorePassword");
			String truststorePassword = truststorePasswordSecret.getValue();

			// Create a PKCS12 KeyStore and load the certificate
			KeyStore trustStore = KeyStore.getInstance("PKCS12");
			trustStore.load(null, truststorePassword.toCharArray());

			CertificateFactory cf = CertificateFactory.getInstance("X.509");
			ByteArrayInputStream certInputStream = new ByteArrayInputStream(certificateBytes);
			X509Certificate x509Certificate = (X509Certificate) cf.generateCertificate(certInputStream);
			certInputStream.close();

			trustStore.setCertificateEntry("http_ca", x509Certificate);

			// Save the truststore to a file
			FileOutputStream truststoreOutputStream = new FileOutputStream(trustLocationStore);
			trustStore.store(truststoreOutputStream, truststorePassword.toCharArray());
			truststoreOutputStream.close();

			System.out.println("Truststore created successfully at: " + trustLocationStore);

			File trustLocationFile = new File(trustLocationStore);

			SSLContextBuilder sslContextBuilder = SSLContexts.custom().loadTrustMaterial(trustLocationFile,
					truststorePassword.toCharArray());
			SSLContext sslContext = sslContextBuilder.build();
			httpClientBuilder.setSSLContext(sslContext);

		} catch (Exception e) {
			System.out.println("Exception Occured");
		}
		return httpClientBuilder;
	}
}